package edu.stanford.nlp.ie.machinereading.domains.bionlp;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class Score {
  static class Counts {
    int correct;
    int predicted;
    int total;
    
    Counts() {
      correct = 0;
      predicted = 0;
      total = 0;
    }
    
    public void add(Counts other) {
      correct += other.correct;
      predicted += other.predicted;
      total += other.total;
    }
    
    public double p() {
      if(predicted == 0) return 0;
      return ((double) correct / (double) predicted);
    }
    
    public double r() {
      if(total == 0) return 0;
      return ((double) correct / (double) total);
    }
    
    public double f1() {
      double p = p();
      double r = r();
      if(p == 0 || r == 0) return 0;
      return (2*p*r/(p + r));
    }
    
    @Override
    public String toString() {
      double p = p();
      double r = r();
      double f1 = f1();
      StringBuffer os = new StringBuffer();
      os.append("P " + pretty(p) + " R " + pretty(r) + " F1 " + pretty(f1) + " " + correct + "/" + predicted + "/" + total);
      return os.toString();
    }
    
    public static double pretty(double v) {
      int times10000 = (int) (v * 10000.00);
      return ((double) times10000) / 100.00;
    }
  };
  
  Map<String, Counts> counts;
  Counts allCounts;

  Score() {
    counts = new HashMap<String, Counts>();
    allCounts = new Counts();
  }
  
  public void add(Score other) {
    allCounts.add(other.allCounts);
    Set<String> keys = other.counts.keySet();
    for(String key: keys) {
      Counts c = counts.get(key);
      if(c == null){
        c = new Counts();
        counts.put(key, c);
      }
      c.add(other.counts.get(key));
    }
  }
  
  public double f1() { return allCounts.f1(); }
  public double p() { return allCounts.p(); }
  public double r() { return allCounts.r(); }
  
  public void incTotal(String label) { incTotal(label, 1); }
  public void incTotal(String label, int value) {
    Counts c = counts.get(label);
    if(c == null){
      c = new Counts();
      counts.put(label, c);
    }
    c.total += value;
    allCounts.total += value;
  }
  
  public void incPredicted(String label) { incPredicted(label, 1); }
  public void incPredicted(String label, int value) {
    Counts c = counts.get(label);
    if(c == null){
      c = new Counts();
      counts.put(label, c);
    }
    c.predicted += value;
    allCounts.predicted += value;
  }
  
  public void incCorrect(String label) { incCorrect(label, 1); }
  public void incCorrect(String label, int value) {
    Counts c = counts.get(label);
    if(c == null){
      c = new Counts();
      counts.put(label, c);
    }
    c.correct += value;
    allCounts.correct += value;
  }
  
  public String toStringShort() {
    return ("P " + Counts.pretty(p()) + " R " + Counts.pretty(r()) + " F1 " + Counts.pretty(f1()));
  }

  @Override
  public String toString() {
    List<String> labels = new ArrayList<String>();
    labels.addAll(counts.keySet());
    Collections.sort(labels);
    StringBuffer os = new StringBuffer();
    for(String label: labels) {
      Counts c = counts.get(label);
      os.append(label + "\t" + c + "\n");
    }
    os.append("---------------------------------------------------------\n");
    os.append("ALL\t" + allCounts + "\n");
    return os.toString();
  }
}
