package edu.stanford.nlp.ie.machinereading.domains.nfl;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @author Andrey Gusev
 * 
 *         Contains type and order of NFL relations which is important for
 *         parsing AMT responses
 * 
 */
public enum NFLRelation {

	TeamScoringAll("teamScoringAll", ARG_TYPE.NFL_TEAM, ARG_TYPE.FINAL_SCORE), TeamFinalScore("teamFinalScore",
			ARG_TYPE.NFL_GAME, ARG_TYPE.FINAL_SCORE), TeamInGame("teamInGame", ARG_TYPE.NFL_TEAM, ARG_TYPE.NFL_GAME), GameWinner(
			"gameWinner", ARG_TYPE.NFL_TEAM, ARG_TYPE.NFL_GAME), GameLoser("gameLoser", ARG_TYPE.NFL_TEAM,
			ARG_TYPE.NFL_GAME), GameDate("gameDate", ARG_TYPE.NFL_GAME, ARG_TYPE.DATE);

	private final String relationType;
	private final String firstArgType;
	private final String secondArgType;

	public static final Map<String, NFLRelation> relationToEnum;

	static {

		Map<String, NFLRelation> temp = new HashMap<String, NFLRelation>();

		for (NFLRelation reation : NFLRelation.values()) {
			temp.put(reation.relationType, reation);
		}

		relationToEnum = Collections.unmodifiableMap(temp);

	}

	private NFLRelation(String relationType, String firstArgType, String secondArgType) {
		this.relationType = relationType;
		this.firstArgType = firstArgType;
		this.secondArgType = secondArgType;
	}

	public String getRelationType() {
		return relationType;
	}

	public String getFirstArgType() {
		return firstArgType;
	}

	public String getSecondArgType() {
		return secondArgType;
	}

	public String getArgType(int argPos) {
		if (argPos == 1) {
			return getFirstArgType();
		} else if (argPos == 2) {
			return getSecondArgType();
		}

		return null;
	}

	static class ARG_TYPE {
		static final String NFL_TEAM = "NFLTeam";
		static final String NFL_GAME = "NFLGame";
		static final String FINAL_SCORE = "FinalScore";
		static final String DATE = "Date";

	}

}
