package edu.stanford.nlp.ie.machinereading.msteventextractor;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import edu.stanford.nlp.ie.machinereading.msteventextractor.DataSet.FilesystemInformation;
import edu.stanford.nlp.util.StringUtils;

public class BioNLP11FilesystemInformation extends FilesystemInformation {
  private static String baseDirectory = "/u/nlp/data/bioNLP/2011/";
  public static String getBaseDirectory() {
    return baseDirectory;
  }

  /**
   * Sets the base directory for all BioNLP 2011 files.  All paths for BioNLP 2011 files will be based on this.
   */
  public static void setBaseDirectory(String baseDirectory) {
    BioNLP11FilesystemInformation.baseDirectory = baseDirectory;
  }

  private String shortName;

  public BioNLP11FilesystemInformation(String shortName, String tokenizer, String syntacticParser) {
    super(tokenizer, syntacticParser);
    this.shortName = shortName;
  }
  
  public void setTokenizer(String tokenizer) {
    Set<String> availableTokenizers = new HashSet<String>(StringUtils.split("default stanford umass"));

    if (!availableTokenizers.contains(tokenizer)) {
      throw new IllegalArgumentException("No such tokenizer: " + tokenizer);
    }
    super.setTokenizer(tokenizer);
  }
  
  public String getTokenizedFilename(String parentPath, String docId) {
    String dirName;
    
    if (parentPath == null) {
      parentPath = baseDirectory;
    }
    
    if (tokenizer.equals("stanford")) {
      dirName = baseDirectory + shortName + "/stanford-tokenizations/";
    } else if (tokenizer.equals("umass")) {
      dirName = baseDirectory + shortName + "/umass-tokenizations/";      
    } else if (tokenizer.equals("default")) { // use included tokenization
      dirName = parentPath + "/tokenised/";
    } else {
      throw new IllegalArgumentException("No such tokenizer: " + tokenizer);
    }
    
    return dirName + docId + ".tok";
  }

  public void setSyntacticParser(String parser) {
    Set<String> availableParsers = new HashSet<String>(StringUtils.split("mccc berkeley enju stanford stanford-mccc umass-mccc"));
    
    if (!availableParsers.contains(parser)) {
      throw new IllegalArgumentException("No such parser: " + parser);
    }
    syntacticParser = parser;
  }

  public String getTreeFilename(String parentPath, String docId) {
    String dirName;
    if (syntacticParser.equals("stanford-mccc")) {
      dirName = baseDirectory + shortName + "/stanford-mccc-parses/";
    } else if (syntacticParser.equals("umass-mccc")) {
      dirName = baseDirectory + shortName + "/umass-mccc-parses/";
    } else {
      dirName = parentPath + File.separator + syntacticParser + "/ptb/";
    }
    
    return dirName + docId + ".ptb";
  }

}
