package edu.stanford.nlp.ie.machinereading.msteventextractor;

import java.io.IOException;

import edu.stanford.nlp.ie.machinereading.msteventextractor.DependencyInstance.SentenceInfo;

public class BioNLPWriter extends DependencyWriter {
  public BioNLPWriter(boolean labeled) {
    this.labeled = labeled;
  }
  
  private String makeDecoderSignature(ParserOptions options) {
    StringBuffer sig = new StringBuffer();
    if(options.secondOrder) sig.append("2");
    else sig.append("1");
    sig.append(options.decodeType.charAt(0));
    return sig.toString().toUpperCase();
  }

  public void write(DependencyInstance instance, ParserOptions options) throws IOException {
    
    String decoderSig = makeDecoderSignature(options);

    for(SentenceInfo sent: instance.sentences){
      // save the original sentence
      writer.write("#SENT:");
      // System.out.println("length = " + instance.originalSentence.length);
      for(int i = 0; i < sent.originalSentence.length; i ++){
        OrigToken t = sent.originalSentence[i];
        writer.write(" ");
        writer.write(t.word);
        writer.write(" ");
        writer.write(t.pos);
        writer.write(" ");
        writer.write(Integer.toString(t.startCharOffset));
        writer.write(" ");
        writer.write(Integer.toString(t.endCharOffset));
      }
      writer.newLine();
      
      // save the parse tree
      writer.write("#TREE: ");
      writer.write(sent.originalTreeAsString);
      writer.newLine();
    }
    
    // save the doc id
    writer.write("#DOC: ");
    writer.write(instance.docId);
    writer.newLine();
        
    if(instance.parseScoresKBest == null){
      // we are in 1-best mode
      saveParse(instance, instance.heads, instance.deprels, -1, 0, 0, decoderSig);
    } else {
      // we are in k-best mode
      for(int k = 0; k < instance.parseScoresKBest.length; k ++){
        saveParse(instance, instance.headsKBest[k], instance.deprelsKBest[k], k, instance.parseScoresKBest[k], instance.nonProjectiveRearrangementsKBest[k], decoderSig);
      }
    }
    writer.newLine();
  }
  
  private void saveParse(DependencyInstance instance, 
      int [] heads, 
      String [] deprels, 
      int rank,
      double parseScore,
      int rearrangements,
      String decoderSig) throws IOException {
    if(rank >= 0){
      writer.write("#PARSE: " + 
          decoderSig + "_RANK=" + (rank + 1) + " " + 
          decoderSig + "_SCORE=" + parseScore + " " +
          decoderSig + "_REARRANGEMENTS=" + rearrangements);
      writer.newLine();
    }
    for (int i = 0; i < instance.length(); i++) {
      String form = instance.forms[i];
      String lemma = instance.lemmas[i];
      writer.write(Integer.toString(i + 1));
      writer.write('\t');
      writer.write(form);
      writer.write('\t');     
      writer.write(lemma == null ? form : lemma);
      writer.write('\t');
      writer.write(instance.postags[i]); // note: this uses postags instead of cpostags since in BioNLP, we don't have both types of POS tags
      writer.write('\t');
      writer.write(instance.postags[i]);
      writer.write('\t');
      writer.write("-");
      writer.write('\t');
      writer.write(Integer.toString(heads[i]));
      writer.write('\t');
      writer.write(deprels[i]);
      writer.write('\t');
      writer.write("-\t-");
      writer.write('\t');
      int startInOriginal = instance.startInOriginal[i];
      int endInOriginal = instance.endInOriginal[i];
      int headInOriginal = instance.headInOriginal[i];
      assert startInOriginal <= headInOriginal;
      assert headInOriginal <= endInOriginal;
      writer.write(Integer.toString(startInOriginal));
      writer.write('\t');
      writer.write(Integer.toString(endInOriginal));
      writer.write('\t');
      writer.write(Integer.toString(headInOriginal));
      writer.write('\t');
      writer.write(instance.ids[i]);
      writer.write('\t');
      writer.write(Integer.toString(instance.sentenceIndex[i]));
      writer.write('\t');
      writer.write(Double.toString(instance.triggerProbs[i]));
      writer.write(',');
      writer.write(Double.toString(instance.firstTriggerProbs[i]));
      writer.newLine();
    }
  }

}
