package edu.stanford.nlp.ie.machinereading.msteventextractor;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import edu.stanford.nlp.util.StringUtils;


public class EpigeneticsDataSet extends DataSet {

  public EpigeneticsDataSet() {
    super();
    
    setFilesystemInformation(new BioNLP11FilesystemInformation("epi", "default", "mccc"));

    entityTypes = new HashSet<String>(StringUtils.split("Protein"));
    eventTypes = new HashSet<String>(StringUtils.split("Hydroxylation Phosphorylation Ubiquitination DNA_methylation Glycosylation Acetylation Methylation Catalysis Deacetylation Deglycosylation Demethylation Deubiquitination Dephosphorylation Dehydroxylation DNA_demethylation"));
    // all events can have at most one theme
    singleThemeEvents = new HashSet<String>(eventTypes);
    eventsWithOnlyThemes = new HashSet<String>(eventTypes);
    eventsWithOnlyThemes.remove("Catalysis");
    eventsRequiringTheme = new HashSet<String>(eventTypes);
    splitBindingEvents = false;
  }

  public void addDomainSpecificFeatures(DependencyPipe pipe, DependencyInstance instance, int headIndex, int childIndex, FeatureVector fv) {
    // none for now
  }

  public void addDomainSpecificLabeledFeatures(DependencyPipe pipe, DependencyInstance instance, int index, String labelName,
      boolean attachingRight, boolean childFeatures, FeatureVector fv) {
    // none for now
  }

  public Map<String, Set<String>> buildTypeGeneralizations() {
    HashMap<String,Set<String>> typeGeneralizations = new HashMap<String, Set<String>>();
    Set<String> simpleEventTypes = new HashSet<String>(StringUtils.split("Hydroxylation Phosphorylation Ubiquitination DNA_methylation Glycosylation Acetylation Methylation Deacetylation Deglycosylation Demethylation Deubiquitination Dephosphorylation Dehydroxylation DNA_demethylation"));
    Set<String> complexEventTypes = new HashSet<String>(StringUtils.split("Catalysis"));
    typeGeneralizations.put("simple", simpleEventTypes);
    typeGeneralizations.put("complex", complexEventTypes);

    // simple + complex = non-protein    
    Set<String> nonProteinEventTypes = new HashSet<String>(simpleEventTypes);
    nonProteinEventTypes.addAll(complexEventTypes);
    typeGeneralizations.put("non-protein", nonProteinEventTypes);

    // degenerate generalization for proteins
    Set<String> proteinEventTypes = new HashSet<String>();
    proteinEventTypes.add("Protein");
    typeGeneralizations.put("protein", proteinEventTypes);
    
    // put de- and their normal forms together into one class
    for (String baseType : StringUtils.split("Acetylation Glycosylation Methylation Ubiquitination Phosphorylation Hydroxylation")) {
      String baseTypeLower = baseType.toLowerCase();
      Set<String> combinedType = new HashSet<String>();
      combinedType.add(baseType);
      combinedType.add("De" + baseTypeLower);
      typeGeneralizations.put(baseTypeLower + "-type", combinedType);
    }
    
    // special case for the above
    Set<String> dnaMethylationType = new HashSet<String>(StringUtils.split("DNA_methylation DNA_demethylation"));
    typeGeneralizations.put("dna_methylation-type", dnaMethylationType);
    
    return typeGeneralizations;
  }

  public boolean includeRole(String role) {
    String roleLower = role.toLowerCase();
    // just the Task 1 roles for now
    return (roleLower.startsWith("theme") || roleLower.startsWith("cause"));
  }
  
}
