///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2007 University of Texas at Austin and (C) 2005
// University of Pennsylvania and Copyright (C) 2002, 2003 University
// of Massachusetts Amherst, Department of Computer Science.
//
// This software is licensed under the terms of the Common Public
// License, Version 1.0 or (at your option) any subsequent version.
// 
// The license is approved by the Open Source Initiative, and is
// available from their website at http://www.opensource.org.
///////////////////////////////////////////////////////////////////////////////

package edu.stanford.nlp.ie.machinereading.msteventextractor;

import java.io.File;
import java.util.regex.Pattern;

/**
 * Hold all the options for the parser so they can be passed around easily.
 * 
 * <p>
 * Created: Sat Nov 10 15:25:10 2001
 * </p>
 * 
 * @author Jason Baldridge
 * @version $Id: CONLLReader.java 103 2007-01-21 20:26:39Z jasonbaldridge $
 * @see mstparser.io.DependencyReader
 */
public final class ParserOptions {

  public String trainfile = null;
  public String testfile = null;
  public String[] stackingtrainfiles = new String[] {};
  public String[] stackingtestfiles = new String[] {};
  public File trainforest = null;
  public File testforest = null;
  public String modelName = "dep.model";
  public String lossType = "punc";
  public String decodeType = "proj";
  public String format = "CONLL";
  public int numIters = 10;
  public String outfile = "out.txt";
  public String goldfile = null;
  public int trainK = 1;
  public int testK = 1;
  public boolean secondOrder = false;
  public boolean useRelationalFeatures = false;
  public boolean discourseMode = false;
  
  public enum conjoiningMethod { NONE, BOTH, DISTANCE_ONLY };
  public conjoiningMethod conjoinSentenceDistance = conjoiningMethod.NONE;
  
  public Pattern featureNamePrefixes = null;
  public boolean collectFeatureCounts = false;
  public boolean featureCountPruning = false;
  public int minFeatureCount = 0;
  public String featureCountsFilename = null;
  public boolean filterOriginalMSTFeatures = false;
  public String edgeFeatureStatisticsFilename = null;
  public String featureNamesFilename = null;

  // class which encapsulates all domain specific information
  public DataSet dataset = null;
  
  public ParserOptions() {
  }
  
  public void makeForest() {
    try {
      @SuppressWarnings("unused")
      File tmpDir = new File("/tmp");
      if (null != trainfile) {
        trainforest = File.createTempFile("train", ".forest");
        trainforest.deleteOnExit();
      }

      if (null != testfile) {
        testforest = File.createTempFile("test", ".forest");
        testforest.deleteOnExit();
      }

    } catch (java.io.IOException e) {
      System.out.println("Unable to create tmp files for feature forests!");
      System.out.println(e);
      System.exit(0);
    }
  }
  
  public void makeFeatureMatchingPattern(String commaSeparatedNames) {
    if(commaSeparatedNames == null) return;
    StringBuilder featurePrefixesRegex = new StringBuilder("^");
    
    if (commaSeparatedNames.equalsIgnoreCase("all")) {
      // leave the regex as just "^" -- this will match everything
    } else {
      int count = 0;
      for (String prefix : commaSeparatedNames.split(",")) {
        if (count > 0) {
          featurePrefixesRegex.append("|");
        }
        featurePrefixesRegex.append("(" + prefix.trim() + ")");
        count++;
      }
    }
    featureNamePrefixes = Pattern.compile(featurePrefixesRegex.toString());
  }
}
