package edu.stanford.nlp.util;

import java.util.*;

/**
 * Contains convenience methods, and utilities for java.util.Iterators
 *
 * @author Huy Nguyen (htnguyen@cs.stanford.edu)
 */
public class Iterators {
  /**
   * Creates an ArrayList containing all of the Objects returned by the given Iterator.
   */
  public static <T> ArrayList<T> asArrayList(Iterator<? extends T> iter) {
    ArrayList<T> al = new ArrayList<T>();
    return (ArrayList<T>) addAll(iter, al);
  }

  /**
   * Creates a HashSet containing all of the Objects returned by the given Iterator.
   */
  public static <T> HashSet<T> asHashSet(Iterator<? extends T> iter) {
    HashSet<T> hs = new HashSet<T>();
    return (HashSet<T>) addAll(iter, hs);
  }

  /**
   * Creates a new Collection from the given CollectionFactory, and adds all of the Objects
   * returned by the given Iterator.
   */
  public static <E> Collection<E> asCollection(Iterator<? extends E> iter, CollectionFactory<E> cf) {
    Collection<E> c = cf.newCollection();
    return addAll(iter, c);
  }

  /**
   * Adds all of the Objects returned by the given Iterator into the given Collection.
   *
   * @return the given Collection
   */
  public static <T> Collection<T> addAll(Iterator<? extends T> iter, Collection<T> c) {
    while (iter.hasNext()) {
      c.add(iter.next());
    }
    return c;
  }

  /**
   * For internal debugging purposes only.
   */
  public static void main(String[] args) {
    String[] test = {"a", "b", "c"};

    List<String> l = Arrays.asList(test);

    System.out.println(asArrayList(l.iterator()));

    System.out.println(asHashSet(l.iterator()));

    System.out.println(asCollection(l.iterator(), CollectionFactory.<String>hashSetFactory()));

    ArrayList<String> al = new ArrayList<String>();
    al.add("d");
    System.out.println(addAll(l.iterator(), al));
  }
}
