package edu.stanford.nlp.util;

import java.util.*;

import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.xpath.XPathEvaluator;
import org.w3c.dom.xpath.XPathNSResolver;
import org.w3c.dom.xpath.XPathResult;
import org.apache.xpath.domapi.XPathEvaluatorImpl;


/** Various utilities for accessing XML documents through XPaths.
 *  (This was separated into its own class, because standard Java 1.5 doesn't
 *  provide XPath functionality.)
 *
 *  @author Bill MacCartney
 *  @author Christopher Manning
 */
public class XPathUtils {

  /**
   */
  public static String getStringFromXPath(String xpath, Node node) {
    List<Node> nodes = getNodesFromXPath(xpath, node);
    StringBuilder b = new StringBuilder();
    for (int i=0; i<nodes.size(); i++) {
      Node n = nodes.get(i);
      b.append(n.getTextContent().trim());
      if (i+1<nodes.size()) b.append(" ");
    }
    return b.toString();
  }

  public static List<Node> getNodesFromXPath(String xpath, Node node) {
    Document doc;
    if (node instanceof Document) {
      doc = (Document) node;
    } else {
      doc = node.getOwnerDocument();
    }
    if (doc==null) throw new RuntimeException();
    // Create an XPath evaluator and pass in the document.
    XPathEvaluator evaluator = new XPathEvaluatorImpl(doc);
    XPathNSResolver resolver = evaluator.createNSResolver(doc);
    // Evaluate the xpath expression
    XPathResult result = (XPathResult)evaluator.evaluate(xpath, node, resolver, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
    List<Node> list = new ArrayList<Node>();
    Node n;
    while ((n = result.iterateNext())!= null) {
      list.add(n);
    }
    return list;
  }
}
